(function () {
  'use strict';

  angular
    .module('app.detalhes')
    .controller('DetalhesController', DetalhesController);

  DetalhesController.$inject = [
    '$scope',
    '$state',
    '$filter',
    '$ionicPopup',
    '$timeout',
    '$translate',
    'UserDataHelper',
    'MathHelper'
  ];

  /* @ngInject */
  function DetalhesController(
    $scope,
    $state,
    $filter,
    $ionicPopup,
    $timeout,
    $translate,
    UserDataHelper,
    MathHelper) {

    var _this = this;

    // Public
    _this.userData = null;

    // Methods
    _this.mostrarFonteCalculo = mostrarFonteCalculo;

    // Constructor
    init();

    /**
     * Inicializa as funcionalidades do controller.
     */
    function init() {
      cordova.plugins.firebase.analytics.logEvent("userEvent", {view: "Detalhes da simulação"});
      var userData = angular.copy(UserDataHelper.get());
      calcularRendimentoPoupanca(userData);
      loadChart(userData);
      _this.userData = userData;
    }

    function calcularRendimentoPoupanca(userData) {
      var rendimentosPoupanca = [];
      var anos = [25, 20, 15, 10, 5];
      for (var a = 0; a < anos.length; a++) {
        var ano = anos[a];
        var valorInicial = userData.tabelaDados[0].fluxoCaixaAnual;
        var montante = [valorInicial];
        for (var i = 1; i <= ano; i++) {
          var tabelaDado = userData.tabelaDados[i];
          montante.push(tabelaDado.fluxoCaixaAnual);
        }
        var npvCalculado = MathHelper.vpl(userData.poupanca, montante);
        var tirCalculado = MathHelper.irr(montante, 0.1);
        rendimentosPoupanca.push({
          ano: ano,
          valor: npvCalculado,
          rentabilidade: tirCalculado
        });
      }
      userData.rendimentosPoupanca = rendimentosPoupanca;
    }

    function loadChart(userData) {
      $timeout(function () {
        var charDataList = [];
        for (var i = 0; i < userData.tabelaDados.length; i++) {
          var tabelaDado = userData.tabelaDados[i];
          charDataList.push({
            'x': i,
            'y': tabelaDado.fluxoCaixaAcumulado
          });
        }
        var data = {
          "xScale": "linear",
          "yScale": "linear",
          "type": "line",
          "main": [{
            "className": ".pizza",
            "data": charDataList
          }]
        };
        var opts = {
          'xMax': 25,
          'xMin': 0,
          'tickHintX': 15,
          'axisPaddingLeft': 10,
          'paddingLeft': 85,
          'tickFormatY': function (y) {
            return $filter('currency')(y, 'R$ ', 2);
          }
        };
        var myChart = new xChart('line', data, '#myChart', opts);
      }, 1000);
    }

    function mostrarFonteCalculo() {
      $translate(['REF_CALC_SYTEM_VALUE', 'ATTENTION'])
        .then(function (translations) {
          alertPopup = $ionicPopup.alert({
            title: translations.ATTENTION,
            template: translations.REF_CALC_SYTEM_VALUE
          });
        });
    }
  }
})();
