(function () {
  "use strict";

  angular.module("app.home").controller("HomeController", HomeController);

  HomeController.$inject = [
    "$cordovaGeolocation",
    "$ionicPopup",
    "$ionicLoading",
    "$state",
    "$cordovaSQLite",
    "$translate",
    "UserDataHelper",
    "DatabaseHelper",
    "$http",
  ];

  /* @ngInject */
  function HomeController(
    $cordovaGeolocation,
    $ionicPopup,
    $ionicLoading,
    $state,
    $cordovaSQLite,
    $translate,
    UserDataHelper,
    DatabaseHelper,
    $http,
  ) {
    var _this = this;
    var alertPopup = null;
    const MAX_INTEGRADORES = 3;

    // Attributes
    _this.estados = [];
    _this.cidades = [];
    _this.userData = null;
    _this.estado = null;
    _this.cidade = null;
    _this.isLoadingRemote = true;
    _this.errorService = false;

    // Methods
    _this.iniciarCalculo = iniciarCalculo;
    _this.onChangeEstado = onChangeEstado;
    _this.onBlurName = onBlurName;
    _this.onBlurEmail = onBlurEmail;
    _this.isValidEmail = isValidEmail;
    _this.tryConnectAgain = tryConnectAgain;

    // Constructor
    init();

    /**
     * Inicializa as funcionalidades do controller.
     */
    function init() {
      _this.userData = {};
      ionic.Platform.ready(function () {
        cordova.plugins.firebase.analytics.logEvent("userEvent", { view: "Dados do cliente" });
        limparBaseDados(function () {
          UserDataHelper.sendPageView('home');
          consultarEstados();
          consultarBaseRemota();
        });
      });
    }

    function limparBaseDados(callback) {
      DatabaseHelper.delete(callback);
    }



    function consultarBaseRemota() {
      $http.get('https://www.weg.net/json/integradores-solar?requestedContent=solarIntegrators')
        .then(function (response) {
          if(response.data && response.data.length > 1) {
            var remoteData = UserDataHelper.getRemoteData();
            remoteData.integradores = response.data.map(obj => ({
              contato: obj.NAME,
              estado: obj.REGION_FULL,
              cidade: obj.CITY,
              endereco: obj.ADDRESS,
              cep: obj.POSTAL_CODE,
              email: obj.EMAIL,
              longitude: obj.GEO_LONGITUDE,
              latitude: obj.GEO_LATITUDE,
              whatsapp: obj.WHATSAPP,
              potencia_2: obj.POWER_2,
              potencia_5: obj.POWER_5,
              potencia_10: obj.POWER_10,
              potencia_30: obj.POWER_30,
              potencia_50: obj.POWER_50,
              potencia_150: obj.POWER_150,
              potencia_acima_150: obj.POWER_ABOVE_150,
              distancia: null,
            }));
          } else {
            _this.errorService = true;
          }
          
          _this.isLoadingRemote = false;

        }, function (error) {
          _this.errorService = true;
          _this.isLoadingRemote = false;
          console.log(error);
        });
    }

    function consultarEstados() {
      if (!ionic.Platform.is("browser")) {
        DatabaseHelper.get(function (db) {
          var query = " SELECT id, ds_nome, sg_uf FROM estado ORDER BY id ASC ";
          $cordovaSQLite.execute(db, query, []).then(
            function (res) {
              console.log('[LOGGER]', "Query success", res);
              var itens = [];
              for (var idx = 0; idx < res.rows.length; idx++) {
                var item = res.rows.item(idx);
                itens.push(item);
              }
              _this.estados = itens;
              console.log('[LOGGER]', "itens", JSON.stringify(itens));
            },
            function (err) {
              console.log('[LOGGER]', "Query error", err);
              _this.estados = [];
            }
          );
        });
      } else {
        console.log('[LOGGER]', "SQL doesn't work on browser mode");
      }
    }

    function consultarCidades(estadoId) {
      if (!ionic.Platform.is("browser")) {
        DatabaseHelper.get(function (db) {
          var query = " SELECT id, ds_nome, sg_uf, nr_latitude, nr_longitude FROM cidade ";
          query += " WHERE id_estado = ? ORDER BY id ASC ";
          $cordovaSQLite.execute(db, query, [estadoId]).then(
            function (res) {
              var itens = [];
              for (var idx = 0; idx < res.rows.length; idx++) {
                var item = res.rows.item(idx);
                itens.push(item);
              }
              _this.cidades = itens;
              console.log('[LOGGER]', "itens", JSON.stringify(itens));
            },
            function (err) {
              _this.cidades = [];
            }
          );
        });
      } else {
        console.log("SQL doesn't work on browser mode");
      }
    }

    function tryConnectAgain() {
      _this.errorService = false;
      _this.isLoadingRemote = true;
      consultarBaseRemota();
    };

    function iniciarCalculo() {
      ionic.Platform.ready(function () {
        $ionicLoading.hide();

        if (alertPopup) {
          alertPopup.close();
        }

        UserDataHelper.clean();

        var userData = UserDataHelper.get();
        userData.name = _this.userData.name;
        userData.email = _this.userData.email;

        if (_this.cidade !== null) {
          cordova.plugins.firebase.analytics.logEvent("userEvent", { action: 'usou_cidade' });
          iniciarCalculoPorCidade(_this.cidade);
        } else {
          cordova.plugins.firebase.analytics.logEvent("userEvent", { action: 'usou_gps' });
          iniciarCalculoPorGPS();
        }
      });
    }

    function iniciarCalculoPorGPS() {

      $ionicLoading.show({
        templateUrl: "templates/loading-gps.html",
        hideOnStateChange: true,
      });

      getCurrentPosition().then(
        function (position) {
          processarLocalizacao(position);
        },
        function (err) {
          tratarErroGps(err.code);
          $ionicLoading.hide();
        }
      );
    }

    function iniciarCalculoPorCidade(cidade) {
      var latitude = cidade.nr_latitude;
      var longitude = cidade.nr_longitude;
      processarLocalizacao({
        coords: {
          latitude: latitude,
          longitude: longitude,
        },
      });
    }

    function getCurrentPosition() {
      var posOptions = {
        timeout: 15000,
        enableHighAccuracy: true,
        maximumAge: 0,
      };
      return $cordovaGeolocation.getCurrentPosition(posOptions);
    }

    function processarLocalizacao(position) {
      var lat = position.coords.latitude;
      var lng = position.coords.longitude;
      getTaxaIrradiacao(lat, lng, function (taxaIrradiacao) {
        if (!taxaIrradiacao) {
          $ionicLoading.hide();
          tratarErroTaxaIrradiacao(lat, lng);
        } else {
          console.log("Taxa de irradiação: " + taxaIrradiacao);
          var userData = UserDataHelper.get();
          userData.estado = _this.estado;
          userData.cidade = _this.cidade;
          userData.latitude = lat;
          userData.longitude = lng;
          userData.irradiacao = taxaIrradiacao;
          getIntegradores(lat, lng, function (integradores) {
            console.log("integradores", JSON.stringify(integradores));
            userData.integradores = integradores;
            console.log('=====> ', JSON.stringify(userData));
            $state.go("calculadora");
          });
        }
      });
    }

    function getTaxaIrradiacao(latitude, longitude, callback) {
      if (ionic.Platform.is("browser")) {
        // fixed irradiacao to test on browser
        callback(4.38);
      } else {
        var params = [latitude, latitude, longitude, longitude];
        DatabaseHelper.get(function (db) {
          var query = " SELECT nr_longitude AS longitude, nr_latitude AS latitude, vl_irradiacao AS irradiacao, ";
          query += " ((? - nr_latitude) * (? - nr_latitude) + ";
          query += " (? - nr_longitude) * (? - nr_longitude)) AS distancia ";
          query += " FROM atlas ORDER BY distancia ASC LIMIT 1 ";
          $cordovaSQLite.execute(db, query, params).then(
            function (res) {
              if (res.rows.length > 0) {
                var row = res.rows.item(0);
                console.log(row);
                callback(row.irradiacao);
              } else {
                callback(null);
              }
            },
            function (err) {
              console.log(err);
              callback(null);
            }
          );
        });
      }
    }

    function getIntegradores(latitude, longitude, callback) {
      if (ionic.Platform.is("browser")) {
        // fixed integradores to test on browser
        callback([
          {
            cidade: "CIDADE 1",
            contato: "CONTATO 1",
            distancia: 0.0012599564149800256,
            endereco: "ENDERECO 1",
            estado: "ESTADO 1",
            cep: "CEP 1",
            email: "EMAIL 1",
            latitude: -26.8909287,
            longitude: -49.0899373,
            potencia_2: 6.0,
            potencia_5: 5.0,
            potencia_10: 4.5,
            potencia_30: 4.0,
            potencia_50: 3.8,
            potencia_150: 3.5,
            potencia_acima_150: 3.35,
          },
          {
            cidade: "CIDADE 2",
            contato: "CONTATO 2",
            distancia: 0.05012776642130124,
            endereco: "ENDERECO 2",
            estado: "ESTADO 2",
            cep: "CEP 2",
            email: "EMAIL 2",
            latitude: -27.0781759,
            longitude: -48.9122007,
            potencia_2: 6.0,
            potencia_5: 5.0,
            potencia_10: 4.5,
            potencia_30: 4.0,
            potencia_50: 3.8,
            potencia_150: 3.5,
            potencia_acima_150: 3.35,
          },
          {
            cidade: "CIDADE 3",
            contato: "CONTATO 3",
            distancia: 0.1556145204500019,
            endereco: "ENDERECO 3",
            estado: "ESTADO 3",
            cep: "CEP 3",
            email: "EMAIL 3",
            latitude: -26.903653,
            longitude: -48.665571,
            potencia_2: 6.0,
            potencia_5: 5.0,
            potencia_10: 4.5,
            potencia_30: 4.0,
            potencia_50: 3.8,
            potencia_150: 3.5,
            potencia_acima_150: 3.35,
          },
        ]);
      } else {
        var remoteData = UserDataHelper.getRemoteData();
        const data = remoteData.integradores;
        if (data && data.length > 0) {
          let integradores = data.map(obj => {
            const integrador = Object.assign({}, obj);
            integrador.distancia = getDistanceFromLatLonInKm(latitude, longitude, integrador.latitude, integrador.longitude);
            return integrador;
          }).sort((a, b) => a.distancia - b.distancia);

          if (_this.cidade !== null) {
            integradores = integradores.slice(0, 10).sort(() => Math.random() - 0.5);
          }

          callback(integradores.slice(0, 4));
        } else {
          callback(null);
        }
      }
    }

    function tratarErroGps(code) {
      $translate(["GPS_ERROR_PERMISSION", "GPS_ERROR_NO_SIGNAL", "GPS_ERROR_TIMEOUT", "GPS_ERROR_UNKNOWN", "ATTENTION"]).then(function (translations) {
        var mensagem = null;
        console.log(code);

        if (code === 1) {
          mensagem = translations.GPS_ERROR_PERMISSION;
        } else if (code === 2) {
          mensagem = translations.GPS_ERROR_NO_SIGNAL;
        } else if (code === 3) {
          mensagem = translations.GPS_ERROR_TIMEOUT;
        } else {
          mensagem = translations.GPS_ERROR_UNKNOWN;
        }

        if (alertPopup) {
          alertPopup.close();
        }

        alertPopup = $ionicPopup.alert({
          title: translations.ATTENTION,
          template: mensagem,
        });
      });
    }

    function tratarErroTaxaIrradiacao(lat, lng) {
      $translate(["IRRADIATION_TAX_ERROR", "ATTENTION"], {
        lat: lat,
        lng: lng,
      }).then(function (translations) {
        var mensagem = translations.IRRADIATION_TAX_ERROR;

        if (alertPopup) {
          alertPopup.close();
        }

        alertPopup = $ionicPopup.alert({
          title: translations.ATTENTION,
          template: mensagem,
        });
      });
    }

    function onChangeEstado() {
      console.log('[LOGGER]', 'onChangeEstado')
      if (_this.estado) {
        console.log('[LOGGER]', 'consultar cidades para ', JSON.stringify(_this.estado));
        consultarCidades(_this.estado.id);
      } else {
        _this.cidades = [];
      }
    }

    function onBlurName() {
      validarNome();
    }

    function onBlurEmail() {
      validarEmail();
    }

    function validarNome() {
      _this.userDataForm.name.$setValidity('valid', true);
      if (!_this.userData.name) {
        _this.userDataForm.name.$setValidity('valid', false);
      }
    }

    function validarEmail() {
      _this.userDataForm.email.$setValidity('valid', true);
      if (!_this.userData.email || !_this.isValidEmail(_this.userData.email)) {
        _this.userDataForm.email.$setValidity('valid', false);
      }
    }

    function isValidEmail(email) {
      var re = /^\w+([\.-]?\w+)*@\w+([\.-]?\w+)*(\.\w{2,3})+$/;
      return email.match(re);
    }

    function getDistanceFromLatLonInKm(lat1, lon1, lat2, lon2) {
      var R = 6371; // Radius of the earth in km
      var dLat = deg2rad(lat2 - lat1);  // deg2rad below
      var dLon = deg2rad(lon2 - lon1);
      var a =
        Math.sin(dLat / 2) * Math.sin(dLat / 2) +
        Math.cos(deg2rad(lat1)) * Math.cos(deg2rad(lat2)) *
        Math.sin(dLon / 2) * Math.sin(dLon / 2)
        ;
      var c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1 - a));
      var d = R * c; // Distance in km
      return d;
    }

    function deg2rad(deg) {
      return deg * (Math.PI / 180)
    }
  }
})();
