(function () {
  "use strict";

  angular.module("app.resultado").controller("ResultadoController", ResultadoController);

  ResultadoController.$inject = [
    "$scope",
    "$state",
    "$filter",
    "$ionicPopup",
    "$cordovaEmailComposer",
    "$cordovaInAppBrowser",
    "$translate",
    "$templateCache",
    "$compile",
    "$timeout",
    "UserDataHelper",
    "MathHelper",
  ];

  /* @ngInject */
  function ResultadoController($scope, $state, $filter, $ionicPopup, $cordovaEmailComposer, $cordovaInAppBrowser, $translate, $templateCache, $compile, $timeout, UserDataHelper, MathHelper) {
    var _this = this;

    // Public
    _this.userData = null;

    // Methods
    _this.detalhesCalculo = detalhesCalculo;
    _this.solicitarOrcamento = solicitarOrcamento;
    _this.solicitarOrcamentoIntegrador = solicitarOrcamentoIntegrador;
    _this.abrirWhatsapp = abrirWhatsapp;
    _this.hasWhatsapp = hasWhatsapp;

    // Constructor
    init();

    /**
     * Inicializa as funcionalidades do controller.
     */
    function init() {
      cordova.plugins.firebase.analytics.logEvent("userEvent", {view: "Resultado da simulação"});
      var userData = angular.copy(UserDataHelper.get());
      console.log('RAW USERDATA', userData);

      // User input
      var irradiacao = userData.irradiacao;
      // var tarifa = userData.conta; // TODO change
      var energia = userData.energia;

      // Calculated values
      var reajuste = MathHelper.percent(4.5);
      var performanceRatio = MathHelper.percent(80);
      var potencia = energia / (irradiacao * 22.5);
      var area = Math.ceil(potencia * 8.0);
      var degradacao = MathHelper.percent(0.5);
      var degradacaoPrimeiroAno = MathHelper.percent(1.1);
      var manutencao = MathHelper.percent(0.5);
      var reajusteOpex = MathHelper.percent(4.5);
      var investimentoTotal = potencia * 5000;
      // var poupanca = Math.pow(1 + MathHelper.percent(0.5), 12) - 1; // 6.17%
      var poupanca = 0.0426; // 4,26%
      var tarifa = userData.conta / userData.energia; // TODO change

      // Update user data
      userData.reajuste = reajuste;
      userData.performanceRatio = performanceRatio;
      userData.potencia = potencia;
      userData.area = area;
      userData.degradacao = degradacao;
      userData.degradacaoPrimeiroAno = degradacaoPrimeiroAno;
      userData.manutencao = manutencao;
      userData.reajusteOpex = reajusteOpex;
      userData.investimentoTotal = investimentoTotal;
      userData.poupanca = poupanca;
      userData.tarifa = tarifa;

      // Generate data table
      gerarTabelaDados(userData);
      calcularPaybackV2(userData);
      gerarMediaIntegradores(userData);

      // Bind to view
      _this.userData = userData;
      UserDataHelper.set(_this.userData);
    }

    function gerarMediaIntegradores(userData) {
      const { potencia, integradores } = userData;
      let mediaIntegradores = null;
      if (integradores) {
        const updatedIntegradores = integradores.map(integrador => {
          const ranges = [
            [0, 2, integrador.potencia_2],
            [2, 5, integrador.potencia_5],
            [5, 10, integrador.potencia_10],
            [10, 30, integrador.potencia_30],
            [30, 50, integrador.potencia_50],
            [50, 150, integrador.potencia_150],
            [150, Number.MAX_VALUE, integrador.potencia_acima_150],
          ];
          console.log("potencia", potencia);
          const range = ranges.find(range => potencia.between(range[0], range[1]));
          console.log("range", range);
          let investimento = potencia * 1000 * range[2];
          console.log("investimento", investimento);
          return {
            ...integrador,
            investimento,
          };
        });
        userData.integradores = updatedIntegradores;
        mediaIntegradores = updatedIntegradores.reduce((acc, integradores) => integradores.investimento + acc, 0) / updatedIntegradores.length;
      }
      userData.mediaIntegradores = mediaIntegradores ? Math.ceil(mediaIntegradores) : null;
    }

    function gerarTabelaDados(userData) {
      var maxAnos = 25;
      var dataArray = initTabelaDados(userData);
      console.log('DATA ARRAY ANTES ITERACAO', dataArray);
      for (var i = 1; i <= maxAnos; i++) {
        calcularIteracao(userData, dataArray, i);
      }
      userData.tabelaDados = dataArray;
    }

    function initTabelaDados(userData) {
      var dataArray = [];
      var geracao = 0.0;
      var tarifa = userData.conta / userData.energia; // TODO change

      console.log('userData.conta', userData.conta);
      console.log('userData.energia', userData.energia);
      console.log('userData.tarifa', tarifa);

      var reembolso = 0.0;
      var capex = userData.investimentoTotal * -1;
      var opex = 0.0;
      var fluxoCaixaAnual = reembolso + capex + opex;
      var fluxoCaixaAcumulado = fluxoCaixaAnual;
      dataArray.push({
        geracao: geracao,
        tarifa: tarifa,
        reembolso: reembolso,
        capex: capex,
        opex: opex,
        fluxoCaixaAnual: fluxoCaixaAnual,
        fluxoCaixaAcumulado: fluxoCaixaAcumulado,
      });
      return dataArray;
    }

    function calcularIteracao(userData, dataArray, i) {
      var past = dataArray[i - 1];
      var geracao = i === 1 ? userData.energia * 12 : past.geracao * (1 - (i === 2 ? userData.degradacaoPrimeiroAno : userData.degradacao));
      var tarifa = past.tarifa * (1 + userData.reajuste); // TODO change
      var reembolso = geracao * tarifa;
      var capex = 0.0;
      var opex = i === 1 ? userData.investimentoTotal * -1 * userData.manutencao : past.opex * (1 + userData.reajusteOpex);
      var fluxoCaixaAnual = reembolso + capex + opex;
      var fluxoCaixaAcumulado = past.fluxoCaixaAcumulado + fluxoCaixaAnual;
      dataArray.push({
        geracao: geracao,
        tarifa: tarifa,
        reembolso: reembolso,
        capex: capex,
        opex: opex,
        fluxoCaixaAnual: fluxoCaixaAnual,
        fluxoCaixaAcumulado: fluxoCaixaAcumulado,
      });
    }

    function calcularPaybackV2(userData) {
      console.log('USERDATA', userData);
      var dataArray = userData.tabelaDados;
      var fcAcumulado = null;
      var fcAcumuladoNextYear = null;
      var investimentoAnos = null;
      for (var i = 1; i <= dataArray.length; i++) {
        if (dataArray[i].fluxoCaixaAcumulado > 0) {
          investimentoAnos = i - 1;
          fcAcumulado = dataArray[i - 1].fluxoCaixaAcumulado;
          fcAcumuladoNextYear = dataArray[i].fluxoCaixaAcumulado;
          break;
        }
      }
      var absFcAcumulado = Math.abs(fcAcumulado);
      var contaMeses = absFcAcumulado / ((absFcAcumulado + fcAcumuladoNextYear) / 12);
      var investimentoMeses = MathHelper.round(contaMeses, 0);
      userData.payback = {
        anos: investimentoAnos,
        meses: investimentoMeses,
      };
    }

    // @Deprecated
    // function calcularPaybackV1(userData) {
    //   var dataArray = userData.tabelaDados;
    //   for (var i = 1; i <= dataArray.length; i++) {
    //     var yearData = dataArray[i];
    //     var fcAcumulado = yearData.fluxoCaixaAcumulado;
    //     var fcAnual = yearData.fluxoCaixaAnual;
    //     if (fcAcumulado > 0) {
    //       var investimentoAnos = i - 1;
    //       var contaMeses = (investimentoAnos * 12) / (investimentoAnos + 1);
    //       var investimentoMeses = MathHelper.round(contaMeses, 0);
    //       userData.payback = {
    //         anos: investimentoAnos,
    //         meses: investimentoMeses
    //       };
    //       return;
    //     }
    //   }
    // }

    function detalhesCalculo() {
      UserDataHelper.set(_this.userData);
      $state.go("detalhes");
    }

    function solicitarOrcamento() { }
    // function solicitarOrcamento() {
    //   var options = {
    //     location: "yes",
    //     clearcache: "yes",
    //     toolbar: "no",
    //   };
    //   $cordovaInAppBrowser.open("http://www.weg.net/br/Contato/Representantes", "_system", options);
    // }

    function hasWhatsapp(integrador) {
      return !!integrador.whatsapp;
    }

    function abrirWhatsapp(integrador) {
      cordova.plugins.firebase.analytics.logEvent("userEvent", {action: 'usou_contato_whatsapp'});
      if (integrador.whatsapp) {
        console.log('========>', integrador.whatsapp);
        var cleanPhone = integrador.whatsapp.replace(/[^+\d]+/g, '');
        // 55 - Brazil fixed
        var options = {
          location: "yes",
          clearcache: "yes",
          toolbar: "no",
        };
        var userData = angular.copy(UserDataHelper.get());
        var factory = angular.element("<div></div>");
        factory.html($templateCache.get("templates/whatsapp.html"));
        $scope.whatsappData = userData;
        $compile(factory)($scope);
        $timeout(function () {
          var msg = factory.text();
          var url = `https://wa.me/55${cleanPhone}?text=${msg}`;
          $cordovaInAppBrowser.open(encodeURI(url), "_system", options);
        });
      } else {
        $translate(["UNAVAILABLE", "ATTENTION"]).then(function (translations) {
          $ionicPopup.alert({
            title: translations.ATTENTION,
            template: translations.UNAVAILABLE,
          });
        });
      }
    }

    function solicitarOrcamentoIntegrador(integrador) {
      cordova.plugins.firebase.analytics.logEvent("userEvent", {action: 'usou_contato_email'});
      console.log("abrir email integrador", integrador);
      cordova.plugins.email.isAvailable(isAvailable => {
        if (isAvailable) {
          enviarEmailOrcamento(integrador);
        } else {
          noEmailClient();
        }
      });
      // $cordovaEmailComposer.isAvailable().then(
      //   function() {
      //     enviarEmailOrcamento();
      //   },
      //   function() {
      //     noEmailClient();
      //   }
      // );
    }

    // @Deprecated
    // function solicitarOrcamento() {
    //   $cordovaEmailComposer.isAvailable()
    //     .then(function() {
    //       enviarEmailOrcamento();
    //     }, function() {
    //       noEmailClient();
    //     });
    // }

    //@Deprecated
    function noEmailClient() {
      cordova.plugins.firebase.analytics.logEvent("userEvent", {action: 'sem_cliente_email'});
      $translate(["NO_EMAIL_APP", "ATTENTION"]).then(function (translations) {
        $ionicPopup.alert({
          title: translations.ATTENTION,
          template: translations.NO_EMAIL_APP,
        });
      });
    }

    function enviarEmailOrcamento(integrador) {
      $translate(["EMAIL_SENDER", "EMAIL_TITLE"]).then(function (translations) {
        var userData = angular.copy(UserDataHelper.get());
        var factory = angular.element("<div></div>");
        factory.html($templateCache.get("templates/email.html"));
        $scope.emailData = userData;
        $compile(factory)($scope);
        $timeout(function () {
          var template = factory.html();
          var email = {
            to: integrador.email,
            subject: translations.EMAIL_TITLE,
            body: template,
            isHtml: true,
          };
          cordova.plugins.email.open(email);
        });
      });
    }

    //@Deprecated
    // function enviarEmailOrcamento() {
    //   $translate(['EMAIL_SENDER', 'EMAIL_TITLE'])
    //     .then(function(translations) {
    //       var userData = angular.copy(UserDataHelper.get());
    //       var factory = angular.element('<div></div>');
    //       factory.html($templateCache.get('templates/email.html'));
    //       $scope.emailData = userData;
    //       $compile(factory)($scope);
    //       $timeout(function() {
    //         var template = factory.html();
    //         var email = {
    //           to: translations.EMAIL_SENDER,
    //           subject: translations.EMAIL_TITLE,
    //           body: template,
    //           isHtml: true
    //         };
    //         $cordovaEmailComposer.open(email).then(null, function() {});
    //       });
    //     });
    // }
  }
})();
