(function () {
  'use strict';

  angular
    .module('app.helper')
    .service('UserDataHelper', UserDataHelper);

  UserDataHelper.$inject = [
    '$translate',
    '$window',
    'LanguagesDomainService'
  ];

  /* @ngInject */
  function UserDataHelper(
    $translate,
    $window,
    LanguagesDomainService) {

    var _this = this;
    var KEY_USER_LOCALE = 'user_lang';
    var mUserData = {};
    var mRemoteData = {};

    _this.clean = function () {
      mUserData = {};
      _this.changeLocale(checkUserLocale());
    };

    _this.get = function () {
      return mUserData;
    };

    _this.set = function (userData) {
      mUserData = userData;
    };

    _this.getRemoteData = function () {
      return mRemoteData;
    };

    _this.setRemoteData = function(remoteData) {
      mRemoteData = remoteData;
    };

    _this.changeLocale = function (locale) {
      storageSet(KEY_USER_LOCALE, locale);
      $translate.use(locale);
    };

    _this.getLocale = function () {
      return checkUserLocale();
    };

    _this.initAnalytics = function () {
      // console.log('STARTING GOOGLE ANALYTICS 2 ====> ', device.uuid);
      // ga('create', 'UA-162899454-1', {
      //   'storage': 'none',
      //   'clientId': device.uuid
      // });
    };

    _this.sendPageView = function (name) {
      // console.log('SENDING PAGE VIEW', name);
      // ga('send', {
      //   hitType: 'pageview',
      //   page: name
      // });
    };

    _this.storageSet = storageSet;
    _this.storageGet = storageGet;

    function checkUserLocale() {
      var locale = storageGet(KEY_USER_LOCALE);
      if (!locale) {
        var browserLang = getBrowserLang();
        if (browserLang && LanguagesDomainService.isSupported(browserLang)) {
          locale = browserLang;
        } else {
          locale = LanguagesDomainService.getDefaultLanguage().key;
        }
        storageSet(KEY_USER_LOCALE, locale);
      }
      return locale;
    }

    function storageSet(key, value) {
      $window.localStorage.setItem(key, JSON.stringify(value));
    }

    function storageGet(key) {
      return JSON.parse($window.localStorage.getItem(key));
    }

    function getBrowserLang() {
      var browserLang = $window.navigator.language;
      if (browserLang && browserLang.indexOf('-') >= 0) {
        return browserLang.split('-')[0];
      }
      return null;
    }
  }

})();
